#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  Copyright 2021-2023 Ramil Nugmanov <nougmanoff@protonmail.com>
#  This file is part of chytorch.
#
#  chytorch is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, see <https://www.gnu.org/licenses/>.
#
import numpy
from pathlib import Path
from setuptools import setup, Extension, find_namespace_packages


setup(
    name='chytorch',
    version='1.48.2',
    packages=find_namespace_packages(include=('chytorch.*',)),
    url='https://github.com/chython/chytorch',
    license='LGPLv3',
    author='Dr. Ramil Nugmanov',
    author_email='nougmanoff@protonmail.com',
    python_requires='>=3.8',
    install_requires=['torchtyping>=0.1.4', 'chython>=1.38', 'torch>=1.8', 'scipy>=1.7', 'numpy>=1.20'],
    setup_requires=['cython'],
    ext_modules=[Extension('chytorch.utils.data.molecule._unpack', ['chytorch/utils/data/molecule/_unpack.pyx'],
                           extra_compile_args=['-O3'], include_dirs=[numpy.get_include()])],
    package_data={'chytorch.zoo': ['README.md'], 'chytorch.utils.data.molecule': ['_unpack.pyx']},
    zip_safe=False,
    long_description=(Path(__file__).parent / 'README.md').read_text('utf8'),
    long_description_content_type='text/markdown',
    classifiers=['Environment :: Plugins',
                 'Intended Audience :: Science/Research',
                 'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python',
                 'Programming Language :: Python :: 3 :: Only',
                 'Programming Language :: Python :: 3.8',
                 'Topic :: Scientific/Engineering',
                 'Topic :: Scientific/Engineering :: Chemistry',
                 'Topic :: Scientific/Engineering :: Information Analysis',
                 'Topic :: Software Development',
                 'Topic :: Software Development :: Libraries',
                 'Topic :: Software Development :: Libraries :: Python Modules']
)
